/*-----------------------------------------------------------------------

File  : clb_pdarrays.h

Author: Stephan Schulz

Contents
 
  Dynamic arrays of pointers and long integers.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Wed Jul 22 21:34:41 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CLB_PDARRAYS

#define CLB_PDARRAYS

#include <clb_memory.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef struct pdarraycell
{
   long   size;
   long   grow;
   IntOrP *array;
}PDArrayCell, *PDArray_p; 


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define PDArrayCellAlloc() (PDArrayCell*)SizeMalloc(sizeof(PDArrayCell))
#define PDArrayCellFree(junk) SizeFree(junk, sizeof(PDArrayCell))

PDArray_p PDArrayAlloc(long init_size, long grow);
PDArray_p PDIntArrayAlloc(long init_size, long grow);
void      PDArrayFree(PDArray_p junk);

IntOrP*   PDArrayElementRef(PDArray_p array, long index);

void      PDArrayElementDeleteP(PDArray_p array, long index);
void      PDArrayElementDeleteInt(PDArray_p array, long index);

#define   PDArrayAssign(array, index, value) \
          *PDArrayElementRef((array), (index)) = (value)
#define   PDArrayAssignP(array, index, value) \
          PDArrayElementRef((array), (index))->p_val = (value)
#define   PDArrayAssignInt(array, index, value) \
          PDArrayElementRef((array), (index))->i_val = (value)

#define   PDArrayElement(array, index) \
	  *PDArrayElementRef((array), (index))
#define   PDArrayElementP(array, index) \
	  (PDArrayElementRef((array), (index))->p_val)
#define   PDArrayElementInt(array, index) \
	  (PDArrayElementRef((array), (index))->i_val)

long      PDArrayMembers(PDArray_p array);
long      PDArrayFirstUnused(PDArray_p array);
long      PDArrayStore(PDArray_p array, IntOrP value);
long      PDArrayStoreP(PDArray_p array, void* value);
long      PDArrayStoreInt(PDArray_p array, long value);
	  

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





