/*-----------------------------------------------------------------------

File  : cco_signals.c

Author: Stephan Schulz

Contents
 
  Signal handler for limit signals...not really necessary, but may
  work around some Solaris bugs.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Nov  6 14:50:28 MET 1998
    New

-----------------------------------------------------------------------*/

#include "cco_signals.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

VOLATILE sig_atomic_t TimeIsUp      = 0;
VOLATILE sig_atomic_t SoftTimeLimit = 0;

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ESignalHandler()
//
//   Handle signals...print message and exit or continue, depending on
//   the signal.
//
// Global Variables: -
//
// Side Effects    : May terminate program or print warning
//
/----------------------------------------------------------------------*/

void ESignalHandler(int mysignal)
{
   struct rlimit     limit = {RLIM_INFINITY, RLIM_INFINITY};

   switch(mysignal)
   {
   case SIGXCPU:
	 if(setrlimit(RLIMIT_CPU, &limit))
	 {
	    TmpErrno = errno;
	    SysError("Unable to reset cpu time limit", SYS_ERROR);
	 }
	 if(SoftTimeLimit)
	 {	  	    
	    TimeIsUp = 1;
	    return;
	 }
	 fprintf(GlobalOut, "# Resource limit exceeded (time)\n");
	 Error("CPU time limit exceeded, terminating", CPU_LIMIT_ERROR);
	 break;
   default:
	 Warning("Unexpected signal caught, continuing");
	 break;
   }
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


